const Ext = imports.misc.extensionUtils.getCurrentExtension();
const Settings = Ext.imports.settings;

let settings = Settings.getSettings();
let buttons = {};

/*
 * name\01icon\01command\02arg1\02arg2\03
 * ↓ ↓ ↓
 * {"name":name,"icon":icon,"command":[command, arg1, arg2]}
 */

function reloadButtons() {
    buttons = {};
    let btnsSerialized = settings.get_string("buttons");
    let btnDataArray = btnsSerialized.split("\x03");
    for (let btnData of btnDataArray) {
        if (btnData == "") continue;
        let button = {};
        let elements = btnData.split("\x01");
        button["name"] = elements[0];
        button["icon"] = elements[1];
        button["command"] = [];
        if (elements.length==3)
            button["command"] = elements[2].split("\x02");
        else
            button["command"] = [];
        buttons[elements[0]] = button;
    }
}

function serializeButtons() {
    let serStr = "";
    for (let name of Object.keys(buttons)) {
        let button = buttons[name];
        serStr += button.name + "\x01"
                + button.icon + "\x01";
        for (let cmd of button.command) {
            serStr += cmd + "\x02";
        }
        serStr = serStr.slice(0,-1) + "\x03";
    }
    settings.set_string("buttons", serStr);
}

function addButton(name, icon, cmd) {
    if (name in buttons) return false;

    reloadButtons();
    let button = {"name": name, "icon": icon, "command": cmd};
    buttons[name] = button;
    serializeButtons();

    return true;
}

function removeButton(name) {
    reloadButtons();
    delete buttons[name];
    serializeButtons();
}

function getButtonNames() {
    reloadButtons();
    return Object.keys(buttons);
}

function getButton(name) {
    reloadButtons();
    return buttons[name];
}
