const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Gtk = imports.gi.Gtk;
const Lang = imports.lang;
const Ext = imports.misc.extensionUtils.getCurrentExtension();
const Buttons = Ext.imports.buttons;

let ui;

const PrefsUI = new Lang.Class({
    Name: "TopBarScriptExecutor",

    _objects: {
        boxMain:        null,
        boxBtns:        null,
        btnAdd:         null,
        btnRem:         null,
        lbScripts:      null,

        addDiag:        null,
        boxAddDiag:     null,
        btnbxAddDiag:   null,
        btnCnfrm:       null,
        btnCncl:        null,
        boxAddDiagCntr: null,
        boxName:        null,
        lblName:        null,
        ntrName:        null,
        boxScript:      null,
        lblScript:      null,
        ntrScript:      null,
        boxIcon:        null,
        lblIcon:        null,
        ntrIcon:        null,
        msgExists:      null,
        btnMsgExistsOk: null,
        lblChanges:     null,
    },

    build: function() {
        this.builder = new Gtk.Builder();
        this.builder.add_from_file(
            Extension.dir.get_child("prefs.ui").get_path());

        for (let o in this._objects) {
            this._objects[o] = this.builder.get_object(o);
        }

        this._initConnections();

        this._objects.addDiag.hide();

        for (let button of Buttons.getButtonNames()) {
            this._createScriptEntry(button);
        }

        return this._objects.boxMain;
    },

    _initConnections: function() {
        this._objects.boxMain.connect("destroy",
            Lang.bind(this, function() { this._objects.msgOnExit.show(); })
        );
        this._objects.addDiag.connect("delete-event",
            Lang.bind(this, function() { this._closeAddDiag(); })
        );

        this._connectBtn("btnAdd", function() {
            this._objects.addDiag.show_now();
        });
        this._connectBtn("btnRem", function() {
            let selected = this._objects.lbScripts.get_selected_row();
            if (selected !== undefined) {
                Buttons.removeButton(selected.get_children()[0].get_text());
                selected.destroy();
            }
        });
        this._connectBtn("btnCnfrm", function() {
            let text = this._objects.ntrName.get_text();
            if (text == "") return;
            for (let row of this._objects.lbScripts.get_children()) {
                if (row.get_children()[0].get_text() == text) {
                    this._objects.msgExists.show_all();
                    return;
                }
            }
            this._createScriptEntry(text);
            Buttons.addButton(
                text,
                this._objects.ntrIcon.get_text(),
                this._objects.ntrScript.get_text().split(" ")
            );
            this._objects.lblChanges.show();
            this._closeAddDiag();
        });
        this._connectBtn("btnCncl", function() {
            this._closeAddDiag();
        });
        this._connectBtn("btnMsgExistsOk", function() {
            this._objects.msgExists.hide();
        });
    },

    _createScriptEntry: function(text) {
        let entry = new Gtk.ListBoxRow();
        entry.add(new Gtk.Label({ label:text }));

        this._objects.lbScripts.insert(entry, -1);
        this._objects.lbScripts.show_all();
    },

    _connectBtn: function(button, func) {
        this._objects[button].connect("clicked",
            Lang.bind(this, func)
        );
    },

    _closeAddDiag: function() {
        this._objects.ntrName.set_text("");
        this._objects.ntrScript.set_text("");
        this._objects.ntrIcon.set_text("");
        this._objects.addDiag.hide();
        return true;
    },
});

function init() {
    ui = new PrefsUI();
}

function buildPrefsWidget() {
    return ui.build();
}
